/*
AvailableFunctions

   Movement  | Counterpart
 -------------------------
|            |            |
|     1      |      8     |
|  forward   |  backward  |
|            |            |
 -------------------------
|            |            |
|     2      |      7     |
|   right    |  rev.right |
|            |            |
 -------------------------
|            |            |
|     3      |      6     |
|   left     |  rev.left  |
|            |            |
 -------------------------
|            |            |
|     4      |      5     |
|  spinRight |  spinLeft  |
|            |            |
 -------------------------

*/

//complicated motor stuff

//Pin 3 to Motor L plus
//Pin 6 to Motor L minus

//Pin 9 to Motor R minus
//Pin 12 to Motor R plus

const int enable1Pin = 7;    // H-bridge enable pin 9
const int motor11Pin = 6;    // H-bridge pin 15
const int motor12Pin = 5;    // H-bridge pin 10

const int enable2Pin = 11;   // H-bridge enable pin 1 
const int motor21Pin = 9;    // H-bridge pin 2
const int motor22Pin = 3;    // H-bridge pin 7

//Variables needed for route creation
int const maxSteps = 3; //how many steps make up one way
int randomNum = 0; //a random number to pick a motor function 
int availableFunctions = 8; //number of available functions for the motor
int randomTime = 0; //int to store movement time of each step

//Arrays to store the TNTs movement
int sequence[maxSteps*2]; //array to store the whole sequence (way there and way back)
int motorMovementTime[maxSteps*2]; //array to store the movement time of the motor 

//Delay
long delayBetweenSteps = 0; //time before new sequence starts


//Include SD card library and RTC library
#include <SD.h>
#include <SPI.h>
#include <Wire.h>
#include "RTClib.h"

//initialize a RTC timer instance
RTC_DS1307 rtc;

//SD card library already makes use of pin 10 which is why it cannot be used and needs to be set as an OUTPUT
const int chipSelect = 10;

String _function;
String _motorMovement;
String _delay;

void setup()
{ 
  randomSeed(analogRead(0));
  
  //set all pins we are using as outputs
  pinMode(enable1Pin, OUTPUT);
  pinMode(motor11Pin, OUTPUT);
  pinMode(motor12Pin, OUTPUT);
  
  pinMode(enable2Pin, OUTPUT);
  pinMode(motor21Pin, OUTPUT);
  pinMode(motor22Pin, OUTPUT);
  
  Serial.begin(9600); //initiate Serial communication for debug
  while (!Serial) {
    ; // wait for serial port to connect. Needed for Leonardo only
  }
  
  Serial.print("Setting up Serial communication... ");
  Serial.println("done.");
  Serial.print("Checking pins... ");
  Serial.println("Pins OK.");
  
  Serial.print("Initializing SD card... ");
  // make sure that the default chip select pin is set to
  // output, even if you don't use it:
  pinMode(10, OUTPUT);
  
  // see if the card is present and can be initialized:
  if (!SD.begin(chipSelect)) {
    Serial.println("Card failed, or not present");
    // don't do anything more:
    return;
  }
  Serial.println("card initialized.");
  
  Serial.print("Preparing RTC timer... ");
  #ifdef AVR
  Wire.begin();
  #else
  Wire1.begin(); // Shield I2C pins connect to alt I2C bus on Arduino Due
  #endif
  rtc.begin();

  if (! rtc.isrunning()) {
    Serial.println("RTC is NOT running!");
    // following line sets the RTC to the date & time this sketch was compiled
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
    // This line sets the RTC with an explicit date & time, for example to set
    // January 21, 2014 at 3am you would call:
    // rtc.adjust(DateTime(2014, 1, 21, 3, 0, 0));
  }
  else {
    Serial.println("RTC running.");
  }
  delay(1000); //intial delay to load program properly
  randomSeed(analogRead(0)); //initialize random number generator with random input - more on http://arduino.cc/en/reference/random
  
  Serial.println("Finished setup. Now GO GO GO! Slowly though... really slow. Almost invisible. Sneaky, so to say...");
  Serial.println("");
  Serial.println("Writing on SD card now...");
  Serial.println("");
}

void loop()
{
  createRandomDelay();
  delay(delayBetweenSteps);
  createRoute(); //create a route for the TNT
  moveIt(); //move TNT
  
  File dataFile = SD.open("movement.txt", FILE_WRITE);
  if (dataFile) {
    dataFile.println("End of sequence");
    dataFile.close();
    Serial.println("End of Sequence");
  } else {
    Serial.println("error opening movement.txt");
  }
}

void createRoute()
{
  for(int i=0; i<maxSteps; i++) //loop creating new route
  {
    //movement
    randomNum = random(1,availableFunctions+1); //get a random function for the motor to perform
    sequence[i] = randomNum; //store function in sequence array
    sequence[((maxSteps*2)-1)-i] = map(randomNum,1,availableFunctions,availableFunctions,1); //store counterpart of function in mirrored position into sequence array
    
    //timing
    randomTime = random(5000, 12001); //get a random movement time for each motor turn
    motorMovementTime[i] = randomTime; //store value in array
    motorMovementTime[((maxSteps*2)-1)-i] = randomTime; //store same value at mirrored position in array
  }

}

void moveIt()
{
  for(int i=0; i<maxSteps; i++) //moving TNT to new location
  { 
    String dataString = "";
    dataString += String(getTimeStamp());
    dataString += " | ";
    dataString += "Delay: ";
    dataString += String(delayBetweenSteps);
    dataString += " | ";
    dataString += "Function: ";
    dataString += String(sequence[i]);
    dataString += " | ";
    dataString += "Time: ";
    dataString += String(motorMovementTime[i]);
    
    File dataFile = SD.open("movement.txt", FILE_WRITE);
    if (dataFile) {
      dataFile.println(dataString);
      dataFile.close();
      Serial.println(dataString);
    }  
    else {
      Serial.println("error opening movement.txt");
    }
    
    callFunction(sequence[i], motorMovementTime[i]); //call specific motor movement
    stopMovement(); //stop between steps
  }
  
  for(int i=maxSteps; i<(maxSteps*2); i++) //moving TNT back to origin
  {
    String dataString = "";
    dataString += String(getTimeStamp());
    dataString += " | ";
    dataString += "Delay: ";
    dataString += String(delayBetweenSteps);
    dataString += " | ";
    dataString += "Function: ";
    dataString += String(sequence[i]);
    dataString += " | ";
    dataString += "Time: ";
    dataString += String(motorMovementTime[i]);
    
    File dataFile = SD.open("movement.txt", FILE_WRITE);
    if (dataFile) {
      dataFile.println(dataString);
      dataFile.close();
      Serial.println(dataString);
    }  
    else {
      Serial.println("error opening movement.txt");
    }
    
    callFunction(sequence[i], motorMovementTime[i]); //call specific motor movement
    stopMovement(); //stop between steps
  }
  
}

void callFunction(int funcNum, int motorDelay) //various TNT movements
{
  switch (funcNum) {
  case 1:
    //Serial.println("forward");
    forward(motorDelay);
    break;
  case 2:
    //Serial.println("right");
    right(motorDelay);
    break;
  case 3:
    //Serial.println("left");
    left(motorDelay);
    break;
  case 4:
    //Serial.println("spinRight");
    spinRight(motorDelay);
    break;
  case 5:
    //Serial.println("spinLeft");
    spinLeft(motorDelay);
    break;
  case 6:
    //Serial.println("rev.left");
    reverseLeft(motorDelay);
    break;
  case 7:
    //Serial.println("rev.right");
    reverseRight(motorDelay);
    break;
  case 8:
    //Serial.println("backward");
    backward(motorDelay);
    break;
  default:
    //don't move
    break;
  }
}

void forward(int _motorDelay) {
  digitalWrite(enable1Pin, HIGH);
  digitalWrite(enable2Pin, HIGH);
  digitalWrite(motor11Pin, HIGH);
  digitalWrite(motor12Pin, LOW);
  digitalWrite(motor21Pin, LOW);
  digitalWrite(motor22Pin, HIGH);
  
  //Serial.print("Motor runs for: ");
  //Serial.println(_motorDelay);
  delay(_motorDelay);
}

void right(int _motorDelay) {
  digitalWrite(enable1Pin, LOW);
  digitalWrite(enable2Pin, HIGH);
  digitalWrite(motor21Pin, LOW);
  digitalWrite(motor22Pin, HIGH);
  
  //Serial.print("Motor runs for: ");
  //Serial.println(_motorDelay);
  delay(_motorDelay);
}

void reverseLeft(int _motorDelay) {
  digitalWrite(enable1Pin, HIGH);
  digitalWrite(enable2Pin, LOW);
  digitalWrite(motor11Pin, LOW);
  digitalWrite(motor12Pin, HIGH);
  
  //Serial.print("Motor runs for: ");
  //Serial.println(_motorDelay);
  delay(_motorDelay);
}

void spinRight(int _motorDelay) {
  digitalWrite(enable1Pin, HIGH);
  digitalWrite(enable2Pin, HIGH);
  digitalWrite(motor11Pin, LOW);
  digitalWrite(motor12Pin, HIGH);
  digitalWrite(motor21Pin, LOW);
  digitalWrite(motor22Pin, HIGH);
  
  //Serial.print("Motor runs for: ");
  //Serial.println(_motorDelay);
  delay(_motorDelay);
}

void spinLeft(int _motorDelay) {
  digitalWrite(enable1Pin, HIGH);
  digitalWrite(enable2Pin, HIGH);
  digitalWrite(motor11Pin, HIGH);
  digitalWrite(motor12Pin, LOW);
  digitalWrite(motor21Pin, HIGH);
  digitalWrite(motor22Pin, LOW);
  
  //Serial.print("Motor runs for: ");
  //Serial.println(_motorDelay);
  delay(_motorDelay);
}

void reverseRight(int _motorDelay) {
  digitalWrite(enable1Pin, LOW);
  digitalWrite(enable2Pin, HIGH);
  digitalWrite(motor21Pin, HIGH);
  digitalWrite(motor22Pin, LOW);
  
  //Serial.print("Motor runs for: ");
  //Serial.println(_motorDelay);
  delay(_motorDelay);
}

void left(int _motorDelay) {
  digitalWrite(enable1Pin, HIGH);
  digitalWrite(enable2Pin, LOW);
  digitalWrite(motor11Pin, HIGH);
  digitalWrite(motor12Pin, LOW);
  
  //Serial.print("Motor runs for: ");
  //Serial.println(_motorDelay);
  delay(_motorDelay);
}

void backward(int _motorDelay) {
  digitalWrite(enable1Pin, HIGH);
  digitalWrite(enable2Pin, HIGH);
  digitalWrite(motor11Pin, LOW);
  digitalWrite(motor12Pin, HIGH);
  digitalWrite(motor21Pin, HIGH);
  digitalWrite(motor22Pin, LOW);
  
  //Serial.print("Motor runs for: ");
  //Serial.println(_motorDelay);
  delay(_motorDelay);
}

String getTimeStamp()
{
  String _date = "";
  
  DateTime now = rtc.now();
  
  _date += String(now.year(), DEC);
  _date += "y - ";
  _date += String(now.month(), DEC);
  _date += "m - ";
  _date += String(now.day(), DEC);
  _date += "d - ";
  _date += String(now.hour(), DEC);
  _date += "h - ";
  _date += String(now.minute(), DEC);
  _date += "m - ";
  _date += String(now.second(), DEC);
  _date += "s";
  
  return _date;
}

void stopMovement()
{
  digitalWrite(enable1Pin, LOW);
  digitalWrite(enable2Pin, LOW);
  digitalWrite(motor11Pin, LOW);
  digitalWrite(motor12Pin, LOW);
  digitalWrite(motor21Pin, LOW);
  digitalWrite(motor22Pin, LOW);
  
  createRandomDelay();  
  delay(delayBetweenSteps);
}

void createRandomDelay()
{
  delayBetweenSteps = random(7200000, 21600000); //20 minutes (1200000), 1h 50min (6600001) 28800000 
  //Serial.print("Time before next step: ");
  //Serial.print(delayBetweenSteps);
  //Serial.println(" ms");
  //Serial.println("");
}
